from unittest.mock import MagicMock

import pytest
from sqlalchemy.ext.asyncio import AsyncEngine

from toolbox.sqlalchemy.connection import DatabaseConnectionSettings, DatabaseConnectionManager


@pytest.fixture
def db_settings():
    class TestSettings(DatabaseConnectionSettings):
        POSTGRES_USER = "postgres"
        POSTGRES_PASSWORD = "postgres"
        POSTGRES_HOST = "0.0.0.0"
        POSTGRES_PORT = "5432"
        POSTGRES_DB = "test_postgres"

    return TestSettings


@pytest.fixture
def mock_engine():
    return MagicMock(spec=AsyncEngine)


@pytest.fixture(scope="function")
def database_connector(db_settings):
    dc = DatabaseConnectionManager(settings=db_settings)
    return dc



from typing import Annotated

from sqlalchemy import select
from fastapi import Depends

from toolbox.sqlalchemy.connection import DatabaseConnectionManager
from toolbox.schemes import SensitiveDataScheme, BaseScheme

AnyPydanticModel = Annotated[BaseScheme, Depends(BaseScheme)]
AnySQLAlchemyModel = Annotated[BaseScheme, Depends(SensitiveDataScheme)]


class AbstractDatabaseCrudManager:
    _alchemy_model = type[AnySQLAlchemyModel]
    _pydantic_model = type[AnyPydanticModel]

    @classmethod
    async def add_one(cls, data_model: AnyPydanticModel) -> None:
        obj = cls._alchemy_model(**data_model.encrypt_fields().model_dump())
        connector = DatabaseConnectionManager()
        session = connector.get_session_maker()
        async with session() as conn:
            conn.add(obj)
            await conn.commit()
            await conn.refresh(obj)

    @classmethod
    async def get_all(cls, limit: int = 100, offset: int = 0):
        session = DatabaseConnectionManager.get_session_maker()
        async with session() as conn:
            query = select(cls._alchemy_model).limit(limit).offset(offset)
            result = [v[0] for v in (await conn.execute(query)).all()]
        return result


import logging
from contextlib import asynccontextmanager


logger = logging.getLogger(__name__)


@asynccontextmanager
async def debug_client(app, app_path: str = "http://test"):
    from httpx import ASGITransport, AsyncClient
    async with AsyncClient(
            transport=ASGITransport(app=app), base_url=app_path,
    ) as client:
        yield client
        pass



from sqlalchemy import select

from tests.fixtures.database import database_connector


async def test_fastapi_depends_itegration_test_2(database_connector):
    from fastapi import Depends, FastAPI
    app = FastAPI()
    @app.get("/")
    async def index(database_conn = Depends(database_connector)):
        async with database_conn:
            res = await database_conn.scalar(select(1))
        return {"status": "ok"}

    from toolbox.testing import debug_client
    async with debug_client(app) as client:
        response1 = await client.get('/')
        assert response1.status_code == 200